/*
 * ADE_TVCSU_ARINC_I2C.c
 *
 *  Created on: 06 Nov 2023
 *      Author: Jigyansu.J
 */

#include "Headers/ADE_TVCSU_HEADERS.h"


I2C_Params i2cParams;
I2C_Handle handle = NULL;
I2C_Transaction i2cTransaction;


void ADE_Tvcsu_i2c_init()
{

    /* Initialize parameters */
    I2C_Params_init(&i2cParams);

    /* Open I2C instance */
    handle = I2C_open(ADE_I2C_INSTANCE , &i2cParams);

}

int I2C_Hub__Read(int i2c_no , uint16_t hub_address , uint16_t hub_ch_no , uint16_t slave_address,uint16_t slave_offset)
{

    I2C_Params i2cParams;
    I2C_Handle handle = NULL;
    I2C_Transaction i2cTransaction;

    i2cParams.bitRate= 0;

    bool status;
    char txBuf[8] = {0x0};
    char rxBuf[8] = {0x0};


    /* Initialize parameters */
    I2C_Params_init(&i2cParams);

    /* Open I2C instance */
    handle = I2C_open(i2c_no , &i2cParams);

    txBuf[0] = (char)((uint32_t) hub_ch_no);

    /* Configure common parameters with I2C transaction */
    i2cTransaction.targetAddress =  hub_address;
    i2cTransaction.writeBuf = (uint8_t *)&txBuf[0];
    i2cTransaction.writeCount = 1;
    i2cTransaction.readBuf = NULL;
    i2cTransaction.readCount = 0;
    i2cTransaction.timeout   = I2C_TRANSACTION_TIMEOUT;

    status = I2C_transfer(handle, &i2cTransaction);
    if (status == false)
    {
        I2C_close(handle);
        I2C_log("\nERROR: I2C_transfer failed");
        return -1;
    }

    memset(rxBuf,0,8);

    txBuf[0] = (char)((uint32_t) slave_offset);

    i2cTransaction.targetAddress = slave_address ;
    i2cTransaction.writeBuf = (uint8_t *)&txBuf[0];
    i2cTransaction.writeCount = 1;
    i2cTransaction.readBuf = (uint8_t *)&rxBuf[0];
    i2cTransaction.readCount = 1;
    i2cTransaction.timeout   = I2C_TRANSACTION_TIMEOUT;
    status = I2C_transfer(handle, &i2cTransaction);
    if (status == false)
    {
        I2C_close(handle);
        I2C_log("\nERROR: I2C_transfer failed");
        return -1;
    }
    else
    {
        for(int k = 0 ; k < 1 ; k++)
        {
//            I2C_log("0x%x ",rxBuf[k]);
        }
    }
    I2C_close(handle);
    return rxBuf[0];
}


int I2C_Hub_Write(int i2c_no , uint16_t hub_address , uint16_t hub_ch_no , uint16_t slave_address,uint16_t slave_offset,uint16_t data)
{
    I2C_Params i2cParams;
    I2C_Handle handle = NULL;
    I2C_Transaction i2cTransaction;

    i2cParams.bitRate= 0;

    bool status;
    char txBuf[8] = {0x0};
    char rxBuf[8] = {0x0};

    /* Initialize parameters */
    I2C_Params_init(&i2cParams);

    /* Open I2C instance */
    handle = I2C_open(i2c_no , &i2cParams);

    txBuf[0] = (char)((uint32_t) hub_ch_no);

    /* Configure common parameters with I2C transaction */
    i2cTransaction.targetAddress = hub_address;
    i2cTransaction.writeBuf = (uint8_t *)&txBuf[0];
    i2cTransaction.writeCount = 1;

    i2cTransaction.readBuf      = NULL;
    i2cTransaction.readCount    = 0;
    i2cTransaction.timeout      = I2C_TRANSACTION_TIMEOUT;

    status = I2C_transfer(handle, &i2cTransaction);
    if (status == false)
    {
        I2C_close(handle);
        I2C_log("\nERROR: I2C_transfer failed in HUB configuration\r\n");
        return -1;
    }
    memset(rxBuf,0,8);

    /* Get board version */
    txBuf[0] = (char)((uint32_t) slave_offset);
    txBuf[1] = (char)((uint32_t) data);

    i2cTransaction.targetAddress = slave_address;
    i2cTransaction.writeBuf = (uint8_t *)&txBuf[0];
    i2cTransaction.writeCount = 2;
    i2cTransaction.readBuf = NULL;
    i2cTransaction.readCount = 0;
    i2cTransaction.timeout   = I2C_TRANSACTION_TIMEOUT;
    status = I2C_transfer(handle, &i2cTransaction);

  //  for(int i=0;i<9999999;i++);

    if (status == false)
    {
        I2C_close(handle);
        I2C_log("\nERROR: I2C_transfer failed in slave write\r\n");
        return -1;
    }
    else
    {
        I2C_log("#");
    }
    I2C_close(handle);
    return 0;
}


int I2C_Write(int i2c_no , uint16_t slave_address,uint16_t slave_offset,uint16_t data)
{
    I2C_Params i2cParams;
    I2C_Handle handle = NULL;
    I2C_Transaction i2cTransaction;

    i2cParams.bitRate= 0;

    bool status;
    char txBuf[8] = {0x0};
    char rxBuf[8] = {0x0};

    /* Initialize parameters */
    I2C_Params_init(&i2cParams);

    /* Open I2C instance */
    handle = I2C_open(i2c_no , &i2cParams);

    txBuf[0] = (char)((uint32_t) slave_offset);
    txBuf[1] = (char)((uint32_t) data);

    /* Configure common parameters with I2C transaction */
    i2cTransaction.targetAddress = slave_address;
    i2cTransaction.writeBuf = (uint8_t *)&txBuf[0];
    i2cTransaction.writeCount = 2;          //1
    i2cTransaction.readBuf = NULL;
    i2cTransaction.readCount = 0;
    i2cTransaction.timeout   = I2C_TRANSACTION_TIMEOUT;

    status = I2C_transfer(handle, &i2cTransaction);
  //  for(int i=0;i<9999999;i++);
    if (status == false)
    {
        I2C_close(handle);
        I2C_log("\nERROR: I2C_transfer failed");
        return -1;
    }
    memset(rxBuf,0,8);

    I2C_log("#");

    I2C_close(handle);
    return 0;
}

int I2C__Read(int i2c_no , uint16_t slave_address,uint16_t slave_offset)
{

    I2C_Params i2cParams;
    I2C_Handle handle = NULL;
    I2C_Transaction i2cTransaction;

    i2cParams.bitRate= 0;

    bool status;
    char txBuf[8] = {0x0};
    char rxBuf[8] = {0x0};

    /* Initialize parameters */
    I2C_Params_init(&i2cParams);

    /* Open I2C instance */
    handle = I2C_open(i2c_no , &i2cParams);

    txBuf[0] = (char)((uint32_t) slave_offset);

    /* Configure common parameters with I2C transaction */
    i2cTransaction.targetAddress =  slave_address;
    i2cTransaction.writeBuf = (uint8_t *)&txBuf[0];
    i2cTransaction.writeCount = 1;
    i2cTransaction.readBuf = (uint8_t *)&rxBuf[0];;
    i2cTransaction.readCount = 1;
    i2cTransaction.timeout   = I2C_TRANSACTION_TIMEOUT;

    status = I2C_transfer(handle, &i2cTransaction);
    if (status == false)
    {
        I2C_close(handle);
        I2C_log("\nERROR: I2C_transfer failed");
        return -1;
    }
#if 1
    for(int k = 0 ; k < 1 ; k++)  //(i2cTransaction.readCount)
    {
        //        I2C_log("0x%x ",rxBuf[k]);
    }
#endif
    I2C_close(handle);
    return rxBuf[0];
}


